package PScm::Read;

use strict;
use warnings;
use PScm::Expr;
use PScm::Token;
use base qw(PScm);

sub new {
    my ($class, $fh) = @_;
    bless {
        FileHandle => $fh,
        Line       => '',
    }, $class;
}

sub Read {
    my ($self) = @_;

    my $token = $self->_next_token();
    return undef unless defined $token;

    if ($token->is_quote_token) {
        my $expr = $self->Read;
        die "syntax Error"
            unless defined($expr) && $expr->is_expr;
        return new PScm::Expr::List(
            $token->is_unquote_token
                ? new PScm::Expr::Symbol('unquote')
                : new PScm::Expr::Symbol('quote'),
            $expr
        );
    }

    if ($token->is_open_token) {
        return $self->read_list();
    } else {
        return $token;
    }
}

sub read_list {
    my ($self) = @_;
    my $token = $self->read_list_element();
    if ($token->is_close_token) {
        return new PScm::Expr::List::Null();
    } elsif ($token->is_dot_token) {
        $token = $self->read_list_element();
        die "syntax error" unless $token->is_expr;
        my $close = $self->read_list_element();
        die "syntax error" unless $close->is_close_token;
        return $token;
    } else {
        return PScm::Expr::List->Cons($token, $self->read_list());
    }
}

sub read_list_element {
    my ($self) = @_;
    my $token = $self->Read();
    die "unexpected EOF"
      if !defined $token;
    return $token;
}

sub _next_token {
    my ($self) = @_;

    while (!$self->{Line}) {
        $self->{Line} = $self->{FileHandle}->getline();
        return undef unless defined $self->{Line};
        $self->{Line} =~ s/^\s+//s;
    }

    for ($self->{Line}) {
        s/^\(\s*// && return PScm::Token::Open->new();
        s/^\)\s*// && return PScm::Token::Close->new();
        s/^\'\s*// && return PScm::Token::Quote->new();
        s/^\,\s*// && return PScm::Token::Unquote->new();
        s/^\.\s*// && return PScm::Token::Dot->new();
        s/^([-+]?\d+)\s*//
          && return PScm::Expr::Number->new($1);
        s/^"((?:(?:\\.)|([^"]))*)"\s*// && do {
            my $string = $1;
            $string =~ s/\\//g;
            return PScm::Expr::String->new($string);
        };
        s/^([^\s\(\)]+)\s*//
          && return PScm::Expr::Symbol->new($1);
    }
    die "can't parse: $self->{Line}";
}

1;
